package net.npaka.fragmentex;
import android.os.Bundle;
import android.app.FragmentTransaction;
import android.app.ListFragment;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.app.Fragment;
import android.view.LayoutInflater;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.TextView;
import android.app.Activity;

//フラグメントの利用
public class FragmentEx extends Activity {
    //アクティビティ起動時に呼ばれる
    @Override
    protected void onCreate(Bundle bundle) {
        super.onCreate(bundle);
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        
        //レイアウトの指定(1)
        setContentView(R.layout.main);
    }
    
    //リストフラグメントの生成(2)
    public static class TitlesFragment extends ListFragment {
        private int pos=-1;

        //アクティビティ生成完了時に呼ばれる(3)
        @Override
        public void onActivityCreated(Bundle bundle) {
            super.onActivityCreated(bundle);
            setListAdapter(new ArrayAdapter<String>(getActivity(),
                android.R.layout.simple_list_item_activated_1, 
                new String[]{"ページ0","ページ1","ページ2"}));
            getListView().setChoiceMode(ListView.CHOICE_MODE_SINGLE);
            showDetails(0);
        }

        //リスト要素クリック時に呼ばれる
        @Override
        public void onListItemClick(ListView l,View v,int position,long id) {
            showDetails(position);
        }

        //詳細の表示
        private void showDetails(int index) {
            getListView().setItemChecked(index,true);
            if (pos==index) return;
            
            //フラグメントの切り換え(4)
            DetailsFragment fragment=DetailsFragment.newInstance(index);
            FragmentTransaction ft=getFragmentManager().beginTransaction();
            ft.replace(R.id.details,fragment);
            ft.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_FADE);
            ft.commit();
            pos=index;
        }
    }
  
    //詳細フラグメント
    public static class DetailsFragment extends Fragment {
        //インスタンス生成時に呼ばれる
        public static DetailsFragment newInstance(int index) {
            DetailsFragment fragment=new DetailsFragment();
            Bundle bundle=new Bundle();
            bundle.putInt("index",index);
            fragment.setArguments(bundle);
            return fragment;
        }

        //フラグメントに関連づけるビュー生成時に呼ばれる(3)
        @Override
        public View onCreateView(LayoutInflater inflater,
            ViewGroup container,Bundle bundle) {
            if (container==null) return null;
            TextView textView=new TextView(getActivity());
            textView.setText("ページ"+
                getArguments().getInt("index",0)+"の詳細");
            textView.setTextSize(24);
            return textView;
        }
    }
}
