package net.npaka.fileex;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.graphics.Color;
import android.os.Bundle;
import android.view.View;
import android.view.Window;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.OutputStream;

//ファイルの読み書き
public class FileEx extends Activity 
    implements View.OnClickListener {
    private final static int WC=LinearLayout.LayoutParams.WRAP_CONTENT;
    private final static int FP=LinearLayout.LayoutParams.FILL_PARENT;
    private EditText editText;//エディットテキスト
    
    //アクティビティ起動時に呼ばれる
    @Override
    public void onCreate(Bundle bundle) {
        super.onCreate(bundle);
        requestWindowFeature(Window.FEATURE_NO_TITLE);

        //レイアウトの生成
        LinearLayout layout=new LinearLayout(this);
        layout.setBackgroundColor(Color.rgb(255,255,255));
        layout.setOrientation(LinearLayout.VERTICAL);
        setContentView(layout);            
        
        //エディットテキストの生成
        editText=new EditText(this);
        editText.setText("",EditText.BufferType.NORMAL);
        editText.setLayoutParams(new LinearLayout.LayoutParams(FP,WC));
        layout.addView(editText);

        //ボタンの生成
        layout.addView(makeButton("書き込み","write"));
        layout.addView(makeButton("読み込み","read"));
    }
    
    //ボタンの生成
    private Button makeButton(String text,String tag) {
        Button button=new Button(this);
        button.setText(text);
        button.setTag(tag);
        button.setOnClickListener(this);
        button.setLayoutParams(new LinearLayout.LayoutParams(WC,WC));
        return button;
    }

    //ボタンクリック時に呼ばれる
    public void onClick(View v) {
        String tag=(String)v.getTag();
        //ファイルへの書き込み
        if (tag.equals("write")) {
            try {
                String str=editText.getText().toString();
                data2file(this,str.getBytes(),"test.txt");//(1)
            } catch (Exception e) {
                showDialog(this,"エラー","書き込み失敗しました ");
            }            
        } 
        //ファイルからの読み込み
        else if (tag.equals("read")) {
            try {
                String str=new String(file2data(this,"test.txt"));//(2)
                editText.setText(str,TextView.BufferType.EDITABLE);        
            } catch (Exception e) {
                showDialog(this,"エラー","読み込み失敗しました ");
            }            
        }
    }     
    
    //バイトデータ→ファイル
    private static void data2file(Context context,
        byte[] w,String fileName) throws Exception {
        OutputStream out=null;
        try {
            //ファイル出力ストリームのオープン(3)
            out=context.openFileOutput(fileName,
                Context.MODE_PRIVATE);
            
            //バイト配列の書き込み(4)
            out.write(w,0,w.length);
            
            //ファイル出力ストリームのクローズ(5)
            out.close();
        } catch (Exception e) {
            try {
                if (out!=null) out.close();
            } catch (Exception e2) {
            }
            throw e;
        }
    } 
    
    //ファイル→バイトデータ
    private static byte[] file2data(Context context,
        String fileName) throws Exception {
        int size;
        byte[] w=new byte[1024]; 
        InputStream in=null;
        ByteArrayOutputStream out=null;
        try {
            //ファイル入力ストリームのオープン(6)
            in=context.openFileInput(fileName);
            
            //バイト配列の読み込み(7)            
            out=new ByteArrayOutputStream();
            while (true) {
                size=in.read(w);
                if (size<=0) break;
                out.write(w,0,size);
            }
            out.close();
            
            //ファイル入力ストリームのクローズ(8)
            in.close();
            
            //ByteArrayOutputStreamオブジェクトのバイト配列化(9)
            return out.toByteArray();
        } catch (Exception e) {
            try {
                if (in!=null) in.close();
                if (out!=null) out.close();
            } catch (Exception e2) {
            }
            throw e;
        }
    }
    
    //ダイアログの表示
    private static void showDialog(Context context,String title,String text) {
        AlertDialog.Builder ad=new AlertDialog.Builder(context);
        ad.setTitle(title);
        ad.setMessage(text);
        ad.setPositiveButton("OK",null);
        ad.show();
    }
}