package net.npaka.sensorex;
import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.graphics.Color;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.view.Window;
import android.widget.LinearLayout;
import android.widget.TextView;
import java.util.List;

//加速度センサーの利用
public class SensorEx extends Activity 
    implements SensorEventListener {
    private final static int WC=LinearLayout.LayoutParams.WRAP_CONTENT;
    private final static float FILTERING=0.1f;
    private TextView      textView;     //テキストビュー
    private SensorManager sensorManager;//センサーマネージャ
    private Sensor        accelerometer;//加速度センサー
    private Sensor        orientation;  //回転センサー
    private float[] values=new float[6];//加速度と傾き値

    //アクティビティ起動時に呼ばれる
    @Override
    public void onCreate(Bundle bundle) {
        super.onCreate(bundle);
        requestWindowFeature(Window.FEATURE_NO_TITLE);

        //レイアウトの生成
        LinearLayout layout=new LinearLayout(this);
        layout.setBackgroundColor(Color.rgb(255,255,255));
        layout.setOrientation(LinearLayout.VERTICAL);
        setContentView(layout); 

        //テキストビューの生成
        textView=new TextView(this);
        textView.setText("SensorEx");
        textView.setTextSize(24.0f);
        textView.setTextColor(Color.rgb(0,0,0));
        textView.setLayoutParams(new LinearLayout.LayoutParams(WC,WC));
        layout.addView(textView);
        
        //センサーマネージャの取得(1)
        sensorManager=(SensorManager)getSystemService(
            Context.SENSOR_SERVICE);
        
        //センサーの取得(2)
        List<Sensor> list;
        list=sensorManager.getSensorList(Sensor.TYPE_ACCELEROMETER);
        if (list.size()>0) accelerometer=list.get(0);
        list=sensorManager.getSensorList(Sensor.TYPE_ORIENTATION);
        if (list.size()>0) orientation=list.get(0);
    }

    //アクティビティ再開時に呼ばれる
    @Override
    protected void onResume() {
        super.onResume();
        
        //センサーの処理の開始(3)
        if (accelerometer!=null) {
            sensorManager.registerListener(this,accelerometer,
                SensorManager.SENSOR_DELAY_NORMAL);
        }
        if (orientation!=null) {
            sensorManager.registerListener(this,orientation,
                SensorManager.SENSOR_DELAY_NORMAL);
        }
    }
    
    //アクティビティ停止時に呼ばれる
    @Override
    protected void onStop() {
        //アプリの停止
        super.onStop();

        //センサーの処理の停止(4)
        sensorManager.unregisterListener(this);
    }
    
    //センサーリスナーの処理(5)
    public void onSensorChanged(SensorEvent event) {
        //加速度の取得
        if (event.sensor==accelerometer) {
            values[0]=(event.values[0]*FILTERING)+(values[0]*(1.0f-FILTERING));
            values[1]=(event.values[1]*FILTERING)+(values[1]*(1.0f-FILTERING));
            values[2]=(event.values[2]*FILTERING)+(values[2]*(1.0f-FILTERING));
        }
        //方向の取得
        if (event.sensor==orientation) {
            values[3]=(event.values[0]*FILTERING)+(values[3]*(1.0f-FILTERING));
            values[4]=(event.values[1]*FILTERING)+(values[4]*(1.0f-FILTERING));
            values[5]=(event.values[2]*FILTERING)+(values[5]*(1.0f-FILTERING));
        }
        //表示
        String text="SensorEx"+
            "\nX軸加速度:"+(int)values[0]+
            "\nY軸加速度:"+(int)values[1]+
            "\nZ軸加速度:"+(int)values[2]+
            "\n方位:"    +(int)values[3]+
            "\nピッチ:"  +(int)values[4]+
            "\nロール:"  +(int)values[5];
        textView.setText(text);
    }

    //精度変更イベントの処理
    public void onAccuracyChanged(Sensor sensor,int accuracy) {
    }
}