package net.npaka.actiongame;
import java.util.Random;
import android.content.*;
import android.graphics.*;
import android.view.*;

//アクションゲーム
public class ActionView extends SurfaceView 
    implements SurfaceHolder.Callback,Runnable {
    //シーン定数(1)
    private final static int 
        S_TITLE   =0,
        S_PLAY    =1,
        S_GAMEOVER=2;
    
    //システム
    private SurfaceHolder holder;
    private Thread        thread;
    private Bitmap[]      bmp=new Bitmap[4];
    private boolean       touchDown;
    
    //シーン
    private int    scene  =S_TITLE;//シーン(1)
    private int    init   =S_TITLE;//初期化(1)
    private String message=null;   //メッセージ
    private int    score  =0;      //スコア
    
    //プレイヤー(3)
    private int     playerY; //Y座標
    private int     jumpPow; //ジャンプ力
    private boolean jumpAble;//ジャンプ可

    //地面(2)
    private int   mapDX;           //X相対位置
    private int[] mapH=new int[10];//高さ
    
    //コンストラクタ
    public ActionView(Context context) {
        super(context);
        
        //ビットマップの読み込み
        for (int i=0;i<4;i++) bmp[i]=readBitmap(context,"r"+i);
        
        //サーフェイスホルダーの生成
        holder=getHolder();
        holder.addCallback(this);
        holder.setFixedSize(800,480);
    }
    
    //サーフェイス生成時に呼ばれる
    public void surfaceCreated(SurfaceHolder holder) {
        init=S_TITLE;
        thread=new Thread(this);
        thread.start();
    }

    //サーフェイス終了時に呼ばれる
    public void surfaceDestroyed(SurfaceHolder holder) {
        thread=null;
    }

    //サーフェイス変更時に呼ばれる
    public void surfaceChanged(SurfaceHolder holder,
        int format,int w,int h) {
    }   
    
    //スレッドの処理
    public void run() {
        Canvas canvas;
        Paint  paint=new Paint();
        while(thread!=null) {
            //初期化
            if (init>=0) {
                scene=init;
                init=-1;
                //タイトル
                if (scene==S_TITLE) {
                    message ="Touch Start";
                    score   =0;
                    playerY =380;
                    jumpPow =0;
                    jumpAble=true;
                    mapDX   =0;
                    for (int i=0;i<10;i++) mapH[i]=1;
                } 
                //プレイ
                else if (scene==S_PLAY) {
                    message  =null;
                    touchDown=false;
                } 
                //ゲームオーバー
                else if (scene==S_GAMEOVER) {
                    message="Game Over";
                }
            }
            
            //プレイ時の処理
            if (scene==S_PLAY) {
                //スコア加算
                score++;

                //衝突判定
                if (playerY>480-mapH[2]*100) {
                    init=S_GAMEOVER;
                    jumpAble=false;
                }
                //上移動
                else if (jumpPow>=0) {
                    playerY-=jumpPow*2;
                    jumpPow--;
                }
                //下移動
                else {
                    playerY+=12;
                    jumpAble=false;
                    if (mapH[2]!=0 &&
                        playerY>480-mapH[2]*100) {
                        playerY=480-mapH[2]*100;
                        jumpAble=true;
                    }
                }
                //スクロール処理(2)
                mapDX-=10;
                if (mapDX==-100) {
                    mapDX=0;

                    //地面の高さを左にシフト
                    System.arraycopy(mapH,1,mapH,0,9);

                    //地面の高さを指定
                    int idx=rand(6);
                    if (mapH[8]==0) {
                        mapH[9]=mapH[7];
                    } else if (idx==0) {
                        mapH[9]=0;
                    } else if (idx==1) {
                        mapH[9]=mapH[8]+1;
                        if (mapH[9]>4) mapH[9]=4;
                    } else if (idx==2) {
                        mapH[9]=mapH[8]-1;
                        if (mapH[9]<1) mapH[9]=1;
                    }
                }

                //イベント
                if (jumpAble) {
                    if (touchDown) {
                        jumpAble=false;
                        jumpPow =14;
                    }
                } else {
                    if (!touchDown) {
                        jumpPow=-10;
                    }
                }
            }
            //ゲームオーバー時の処理
            else if (scene==S_GAMEOVER) {
                if (playerY<700) playerY+=16;
            }
            
            //背景の描画
            canvas=holder.lockCanvas();
            canvas.drawBitmap(bmp[0],0,0,null);
            
            //プレイヤーの描画
            int idx=3;
            if (jumpAble) idx=2+score%2;
            canvas.drawBitmap(bmp[idx],150,playerY-100,null);

            //地面の描画
            for (int i=0;i<mapH.length;i++) {
                canvas.drawBitmap(bmp[1],mapDX+i*100,
                    480-mapH[i]*100,null);
            }
            
            //スコアの描画
            paint.setColor(Color.WHITE);
            paint.setTextSize(30);
            canvas.drawText("SCORE "+num2str(score,6),10,40,paint);
            
            //メッセージの描画
            if (message!=null) {
                canvas.drawText(message,
                    (800-paint.measureText(message))/2,100,paint);
            }
            holder.unlockCanvasAndPost(canvas);
            
            //スリープ
            try {
                Thread.sleep(50);
            } catch (Exception e) {
            }
        }
    }
    
    //タッチイベントの処理
    @Override
    public boolean onTouchEvent(MotionEvent event) {
        int action=event.getAction();
        switch (action&MotionEvent.ACTION_MASK) {
            case MotionEvent.ACTION_DOWN:
                if (scene==S_TITLE) {
                    init=S_PLAY;
                } else if (scene==S_PLAY) {
                    touchDown=true;
                } else {
                    if (playerY>=700) init=S_TITLE;
                }
                break;
            case MotionEvent.ACTION_UP:
                if (scene==S_PLAY) {
                    touchDown=false;
                }
                break;
        }
        return true;
    }
    
    //乱数の取得
    private static Random rand=new Random();
    private static int rand(int num) {
        return (rand.nextInt()>>>1)%num;
    }
    
    //数値→文字列
    private static String num2str(int num,int len) {
        String str=""+num;
        while(str.length()<len) str="0"+str;
        return str;
    }
    
    //ビットマップの読み込み
    private static Bitmap readBitmap(Context context,String name) {
        int resID=context.getResources().getIdentifier(
            name,"drawable",context.getPackageName());
        return BitmapFactory.decodeResource(
            context.getResources(),resID);
    }
}