package net.npaka.blockgame;
import java.util.Random;

import android.content.*;
import android.graphics.*;
import android.view.*;

//ブロック崩し
public class BlockView extends SurfaceView 
    implements SurfaceHolder.Callback,Runnable {
    //シーン定数(1)
    private final static int 
        S_TITLE   =0,
        S_PLAY    =1,
        S_CLEAR   =2,
        S_GAMEOVER=3;
    
    //ブロック定数(4)
    private final static int[] BLOCK={
        0,0,1,1,0,
        1,0,0,0,1,
        0,0,0,1,0,
        0,1,0,0,0,
        0,1,1,0,0,
        0,0,1,1,0,
        1,0,0,0,1,
        1,1,1,0,0,
        1,0,1,0,0,
        0,0,0,1,1,
        0,0,1,0,0,
        1,0,1,1,0,
        1,0,1,0,0,
        0,1,0,0,1};
    
    //システム
    private SurfaceHolder holder;
    private Thread        thread;
    private Bitmap[]      bmp=new Bitmap[4];
    
    //シーン
    private int    scene  =S_TITLE;//シーン(1)
    private int    init   =S_TITLE;//初期化(1)
    private String message=null;   //メッセージ
    private int    score  =0;      //スコア
    
    //ブロック
    private int[] block=new int[70];//ブロック

    //ボール・バー
    private int ballX =0;//ボールX座標
    private int ballY =0;//ボールY座標
    private int ballVX=0;//ボールX速度
    private int ballVY=0;//ボールY速度
    private int barX  =0;//バーX座標    
    
    //コンストラクタ
    public BlockView(Context context) {
        super(context);
        
        //ビットマップの読み込み
        for (int i=0;i<4;i++) bmp[i]=readBitmap(context,"r"+i);
        
        //サーフェイスホルダーの生成
        holder=getHolder();
        holder.addCallback(this);
        holder.setFixedSize(480,800);
    }
    
    //サーフェイス生成時に呼ばれる
    public void surfaceCreated(SurfaceHolder holder) {
        init=S_TITLE;
        thread=new Thread(this);
        thread.start();
    }

    //サーフェイス終了時に呼ばれる
    public void surfaceDestroyed(SurfaceHolder holder) {
        thread=null;
    }

    //サーフェイス変更時に呼ばれる
    public void surfaceChanged(SurfaceHolder holder,
        int format,int w,int h) {
    }   
    
    //スレッドの処理
    public void run() {
        Canvas canvas;
        Paint  paint=new Paint();
        
        //定期処理(2)
        while(thread!=null) {
            //シーンの初期化
            if (init>=0) {
                scene=init;
                init=-1;
                //タイトルの初期化
                if (scene==S_TITLE) {
                    message="Touch Start";
                    for (int i=0;i<block.length;i++) {
                        block[i]=BLOCK[i];
                    }
                    ballX =240;
                    ballY =650-14;
                    ballVX=0;
                    ballVY=0;
                    barX  =240;
                } 
                //プレイの初期化
                else if (scene==S_PLAY) {
                    ballVX= 4;
                    ballVY=-4;
                    message=null;
                } 
                //クリアの初期化
                else if (scene==S_CLEAR) {
                    message="Clear";
                } 
                //ゲームオーバーの初期化
                else if (scene==S_GAMEOVER) {
                    message="Game Over";
                }
            }
            
            //ボールの移動
            if (scene==S_PLAY) {
                for (int j=0;j<4;j++) moveBall();
            }
            
            //背景の描画
            canvas=holder.lockCanvas();
            canvas.drawBitmap(bmp[0],0,0,null);
            
            //スコアの描画
            paint.setColor(Color.WHITE);
            paint.setTextSize(30);
            canvas.drawText("SCORE "+num2str(score,4),10,40,paint);
            
            //ブロックの描画
            int num=0;
            for (int i=0;i<BLOCK.length;i++) {
                if (block[i]!=0) {
                    canvas.drawBitmap(bmp[1],
                        (i%5)*96,62+(i/5)*32,null);
                    num++;
                }
            }
            if (scene==S_PLAY && num==0) init=S_CLEAR;
            
            //バーとボールの描画
            canvas.drawBitmap(bmp[3],barX-60,650,null);
            canvas.drawBitmap(bmp[2],ballX-14,ballY-14,null);
            
            //メッセージの描画
            if (message!=null) {
                canvas.drawText(message,
                    (480-paint.measureText(message))/2,450,paint);
            }
            holder.unlockCanvasAndPost(canvas);
            
            //スリープ
            try {
                Thread.sleep(40);
            } catch (Exception e) {
            }
        }
    }
    
    //ボール移動
    private void moveBall() {
        ballX+=ballVX;
        ballY+=ballVY;
                
        //周囲との衝突判定
        if (ballX<0)   {ballX=  0;ballVX=-ballVX;}
        if (480<ballX) {ballX=480;ballVX=-ballVX;}
        if (ballY<62)  {ballY= 62;ballVY=-ballVY;}
        if (ballY>820) init=S_GAMEOVER;
                
        //ブロックとの衝突判定
        for (int i=0;i<BLOCK.length;i++) {
            if (block[i]==0) continue;
            int r=hitLineRect(
                ballX-ballVX,ballY-ballVY,ballX,ballY,
                (i%5)*96,50+(i/5)*32,96,32);
            if (r>=0) {
                if (r==0 || r==1) ballVY=-ballVY;
                if (r==2 || r==3) ballVX=-ballVX;
                block[i]=0;
                score+=10;
                break;
            }
        }
                
        //バーとの衝突判定
        int r=hitLineRect(
            ballX-ballVX,ballY-ballVY,ballX,ballY,
            barX-60,650,120,30);
        if (r>=0) {
            if (r==0) ballVY=-4-rand(2);
            if (r==1) ballVY=-ballVY;
            if (r==2 || r==3) ballVX=-ballVX;
        }        
    }
    
    //タッチイベントの処理
    @Override
    public boolean onTouchEvent(MotionEvent event) {
        int action=event.getAction();
        switch (action&MotionEvent.ACTION_MASK) {
            case MotionEvent.ACTION_DOWN:
                if (scene==S_TITLE) {
                    init=S_PLAY;
                } else if (scene==S_PLAY) {
                    barX=(int)event.getX();
                    score=0;
                } else {
                    init=S_TITLE;
                }
                break;
            case MotionEvent.ACTION_MOVE:
                if (scene==S_PLAY) {
                    barX=(int)event.getX();
                }
                break;
        }
        return true;
    }
    
    //直線と矩形の衝突判定
    private static int hitLineRect(
        int x0,int y0,int x1,int y1,
        int x2,int y2,int w2,int h2) {
        if (x2<=x1 && x1<=x2+w2 && y2<=y1 && y1<=y2+h2) {
            if (hitLineLine(x0,y0,x1,y1,x2,   y2,   x2+w2,y2   )) return 0;//上
            if (hitLineLine(x0,y0,x1,y1,x2,   y2+h2,x2+w2,y2+h2)) return 1;//下
            if (hitLineLine(x0,y0,x1,y1,x2,   y2,   x2,   y2+h2)) return 2;//左
            if (hitLineLine(x0,y0,x1,y1,x2+w2,y2   ,x2+w2,y2+h2)) return 3;//右
        }
        return -1;
    }
    
    //2つの直線の衝突判定
    private static boolean hitLineLine(
        float x0,float y0,float x1,float y1,
        float x2,float y2,float x3,float y3) {
        float r=((y3-y2)*(x2-x0)-(x3-x2)*(y2-y0))/((x1-x0)*(y3-y2)-(y1-y0)*(x3-x2));
        float s=((y1-y0)*(x2-x0)-(x1-x0)*(y2-y0))/((x1-x0)*(y3-y2)-(y1-y0)*(x3-x2));
        return (0<r && r<=1 && 0<s && s<=1);
    }
    
    //乱数の取得(3)
    private static Random rand=new Random();
    private static int rand(int num) {
        return (rand.nextInt()>>>1)%num;
    }
    
    //数値→文字列
    private static String num2str(int num,int len) {
        String str=""+num;
        while(str.length()<len) str="0"+str;
        return str;
    }
    
    //ビットマップの読み込み
    private static Bitmap readBitmap(Context context,String name) {
        int resID=context.getResources().getIdentifier(
            name,"drawable",context.getPackageName());
        return BitmapFactory.decodeResource(
            context.getResources(),resID);
    }
}